// c3dtest.cpp : Defines the entry point for the console application.
//

//#include "stdafx.h"
#include "basic_io.hpp"
#include "uuc3d.hpp"
#include <iostream>

using UuIcsC3d::Content;
using UuIcsC3d::BasicIO;
using std::cout;
using std::vector;
using std::string;

void list_header(Content const &ct)
{
    boost::shared_ptr<BasicIO> biop=ct.io();
    UuIcsC3d::Header const &hd = ct.header();
    //cout<<"First parameter: "<<static_cast<unsigned>(hd[0])<<"\n";
    cout<<"Trajectory count: "<<hd.trajectory_count<<"\n";
    cout<<"Analog data: "<<hd.analog_per_frame<<"\n";
    cout<<"First frame: "<<hd.first_frame<<"\n";
    cout<<"Last frame: "<<hd.last_frame<<"\n";
    cout<<"Video sampling rate: "<<hd.frame_rate<<"\n";
    cout<<"Analog sampling rate: "<<hd.analog_samples_per_frame<<"\n";
    float scalefac = hd.scaling_factor;
    cout<<"Scale factor: "<<scalefac<<"\n";
    cout<<"Start record: "<<hd.data_start_block<<"\n";
    cout<<"Maximum interpolation gap: "<<hd.max_interpolation_gap<<"\n";
    if (hd.has_label_and_range) {
        cout<<"First Label block: "<<hd.label_and_range_block<<"\n";
    }
    int event_count = hd.events.size();
    cout<<"Event count: "<<event_count<<"\n";
    
    UuIcsC3d::Group const*group;
    if ((group=ct.get_group_checked("TRIAL"))!=0) {
        UuIcsC3d::Parameter const *param;
        if ((param=group->get_parameter_checked("ACTUAL_START_FIELD"))!=0) {
			unsigned int value = param->get_ushort(1);
			value = value<<16;
			value += param->get_ushort(0);
			cout<<"TRIAL:ACTUAL_START_FIELD: "<<value<<"\n";
        }
        if ((param=group->get_parameter_checked("ACTUAL_END_FIELD"))!=0) {
			unsigned int value = param->get_ushort(1);
			value = value<<16;
			value += param->get_ushort(0);
			cout<<"TRIAL:ACTUAL_END_FIELD:"<<value<<"\n";
        }
    }
}

void test1(string const &filename)
{
	cout<<"\n***** Header test *****\n";
    Content ct = UuIcsC3d::open_c3d_file(filename);
    list_header(ct);
}

void test1a(string const &filename)
{
	using UuIcsC3d::Group;
	using UuIcsC3d::Parameter;
	cout<<"\n***** Parameter test *****\n";
    Content ct = UuIcsC3d::open_c3d_file(filename);
	vector<Group> const &groups=ct.the_groups();
	for (vector<Group>::const_iterator cur = groups.begin(); cur!=groups.end(); ++cur) {
		cout<<"Group:\t"<<cur->id()<<"\t"<<cur->name()<<"\t"<<cur->description()<<"\n";
		vector<Parameter> const &parameters = cur->the_parameters();
		for (vector<Parameter>::const_iterator par = parameters.begin(); par!=parameters.end(); ++par) {
			cout<<"  Parameter:\t"<<par->name()<<"\t"<<par->data_type()<<"\t"<<par->description()<<"\n";
		}
	}
}

void test2(string const &filename)
{
	cout<<"\n***** Test 2 *****\n";
	UuIcsC3d::C3dFileInfo fi(filename);
	if (fi.points_per_frame() <= 0)
		return;
	int fc = fi.frame_count();
	cout<<fc<<"\n";
	cout<<fi.points_per_frame()<<"\n";
	for (size_t i=0; i<fi.point_labels().size(); ++i) {
		cout<<i<<" "<<fi.point_labels()[i].complete()<<"|\n";
	}
	std::auto_ptr<UuIcsC3d::C3dFile> filep(fi.open());
	vector<float> x1(fc);
	vector<float> x2(fc);
	UuIcsC3d::FrameData frame_data;
	for (int i=0; i<fc; ++i) {
		filep->get_frame_data(frame_data, i);
		UuIcsC3d::DataPoint3d const &dp=frame_data.points[0];
		if (!dp.is_valid())
			cout<<"Point 0 in frame "<<i<<"is invalid.\n";
		x1[i] = dp.x();
	}
	for (int i=fc; i>0; --i) {
		x2[i-1] =filep->get_point(i-1, 0).x();
	}
	for (int i=0; i<fc; ++i) {
		if (x1[i]!=x2[i])
			cout<<x1[i]-x2[i]<<"\t"<<x1[i]<<"\n";
	}
}

void test3(string const &filename_in)
{
	cout<<"\n***** Test 3 *****\n";
	UuIcsC3d::C3dFileInfo fi(filename_in);
	int fc = fi.frame_count();
	std::auto_ptr<UuIcsC3d::C3dFile> filep(fi.open());
	vector< UuIcsC3d::FrameData > frame_data(fc);
	for (int i=0; i<fc; ++i) {
		filep->get_frame_data(frame_data[i], i);
	}
	filep.reset();
	if (frame_data.size()>0)
		frame_data.pop_back();
	UuIcsC3d::write_with_same_header("out_f.c3d", fi, frame_data);
//	UuIcsC3d::write_f("out_f.c3d", fi.content(), frame_data);
//	UuIcsC3d::write_i("out_i.c3d", fi.content(), frame_data);
}

void test4(string const &filename_in, string const &filename_out)
{
	cout<<"\n***** Copy test *****\n";
	UuIcsC3d::C3dFileInfo fi(filename_in);
	int fc = fi.frame_count();
	std::auto_ptr<UuIcsC3d::C3dFile> filep(fi.open());
	vector< UuIcsC3d::FrameData > frame_data(fc);
	for (int i=0; i<fc; ++i) {
		filep->get_frame_data(frame_data[i], i);
	}
	filep.reset();
//	if (frame_data.size()>0)
//		frame_data.pop_back();
	UuIcsC3d::write(filename_out, fi, frame_data, UuIcsC3d::get_native_io().get());
}


void test5(string const &filename_in, string const &filename_out)
{
	cout<<"\n***** Add trajectory test *****\n";
	UuIcsC3d::C3dFileInfo fi(filename_in);
	int fc = fi.frame_count();
	if (fc==0) {
		std::cerr<<"No copy made because no frames were found in input.\n";
		return;
	}
	std::auto_ptr<UuIcsC3d::C3dFile> filep(fi.open());
	vector< UuIcsC3d::FrameData > frame_data(fc);
	for (int i=0; i<fc; ++i) {
		filep->get_frame_data(frame_data[i], i);
	}
	filep.reset();

	// Add a point label by:
	{
		// - copy the stripped labels
		typedef vector<UuIcsC3d::SpacePaddedString> Psv;
		Psv padded_labels = fi.point_labels();
		if (padded_labels.size() != frame_data[0].points.size()) {
			std::cerr<<"Number of labels is not equal to number of trajectories.\n";
		}
		vector<string> labels;
		for (Psv::const_iterator cur = padded_labels.begin(); cur!=padded_labels.end(); ++cur) {
			labels.push_back(cur->stripped());
		}
		// - add a label
		labels.push_back("NewLabel");
		// - set the new labels
		fi.set_point_labels(labels);
	}

	// Add a new trajectory (point)
	{
		using UuIcsC3d::DataPoint3d;
		// Create an invalid data point
		DataPoint3d dp;
		// Add a point to every frame
		for (int i=0; i<fc; ++i) {
			vector<DataPoint3d> &points = frame_data[i].points;
			//dp.set_value(0.1f, 0.4f, 11.2f, DataPoint3d::VirtualMarker);
			points.resize(points.size()+1, dp);
		}
	}

	UuIcsC3d::write(filename_out, fi, frame_data, UuIcsC3d::get_native_io().get());
}



int main(int argc, char* argv[])
{
	if (argc<3)
		return 2;
//    test1(argv[1]);
//	test1a(argv[1]);
//    test2(argv[1]);
//    test3(argv[1]);
    test4(argv[1], argv[2]);
	return 0;
}

