// vim:set tabstop=4 cindent:

// Author: Geert-Jan Giezeman <geert@cs.uu.nl>
// Copyright: Utrecht University, 2011

#include "basic_io.hpp"
#include <algorithm>

using UuIcsC3d::BasicIO;

void BasicIO::encode_array(unsigned char *buf, short int const *in_arr, int n ) const
{
	for (int i=0; i<n; ++i) {
		encode(buf, *in_arr++);
		buf += 2;
	}
}

void BasicIO::encode_array(unsigned char *buf, unsigned short const *in_arr, int n) const
{
	for (int i=0; i<n; ++i) {
		encode(buf, *in_arr++);
		buf += 2;
	}
}

void BasicIO::encode_array(unsigned char *buf, float const *in_arr, int n) const
{
	for (int i=0; i<n; ++i) {
		encode(buf, *in_arr++);
		buf += 4;
	}
}

namespace {
    struct IntelIO: BasicIO {
		UuIcsC3d::EncodingType get_encoding() const { return UuIcsC3d::Pt_Intel;}
		unsigned short to_uint16(unsigned char const *inbuf) const;
		unsigned int to_uint32(unsigned char const *inbuf) const;
        short to_int16(unsigned char const *) const;
		int to_int32(unsigned char const *inbuf) const;
        float to_float(unsigned char const *) const;
        void encode(unsigned char *, short) const;
        void encode(unsigned char *, unsigned short) const;
        void encode(unsigned char *, int) const;
        void encode(unsigned char *, unsigned int) const;
        void encode(unsigned char *, float) const;
    };
/*
	uint64_t byteArrayToUInt64 (unsigned char *arr, size_t bytes, int type)
{
uint64_t num = UINT64_C (0);
uint64_t tmp;

size_t i;
for (i = 0; i < bytes; i  )
 {
   tmp = UINT64_C (0);
   if (type == LITTLE)
     tmp = arr[i];
   else if (type == BIG)
     tmp = arr[bytes - i - 1];

 num |= (tmp << ((i & 7) << 3));
 }

return num;
}
*/
	unsigned int IntelIO::to_uint32(unsigned char const *inbuf) const
	{
		unsigned int num = 0;
		for (size_t i = 0; i < 4; ++i  )
		{
			unsigned int tmp = inbuf[i];
			num |= (tmp << (i << 3));
		}
		return num;
	}


	unsigned short IntelIO::to_uint16(unsigned char const *inbuf) const
	{
		unsigned short num = 0;
		for (size_t i = 0; i < 2; ++i  )
		{
			unsigned short tmp = inbuf[i];
			num |= (tmp << (i << 3));
		}
		return num;
	}

    short IntelIO::to_int16(unsigned char const *inbuf) const
    {
		unsigned short val1 = to_uint16(inbuf);
		return reinterpret_cast<short &>(val1);
    }

    int IntelIO::to_int32(unsigned char const *inbuf) const
    {
		unsigned val1 = to_uint32(inbuf);
		return reinterpret_cast<int &>(val1);
    }

    float IntelIO::to_float(unsigned char const *inbuf) const
    {
		unsigned val1 = to_uint32(inbuf);
		return reinterpret_cast<float &>(val1);
    }
	void IntelIO::encode(unsigned char *obuf, unsigned short value) const
    {
		for (int i=0; i<2; ++i) {
			*obuf++ = static_cast<unsigned char>(value &0xff);
			value = value >> 8;
		}
    }
	void IntelIO::encode(unsigned char *obuf, unsigned int value) const
    {
		for (int i=0; i<4; ++i) {
			*obuf++ = static_cast<unsigned char>(value &0xff);
			value = value >> 8;
		}
    }
    void IntelIO::encode(unsigned char *obuf, short value) const
    {
        unsigned short sval = reinterpret_cast<unsigned short &>(value);
		encode(obuf, sval);
    }
    void IntelIO::encode(unsigned char *obuf, int value) const
    {
        unsigned int sval = reinterpret_cast<unsigned int &>(value);
		encode(obuf, sval);
    }
    void IntelIO::encode(unsigned char *obuf, float value) const
    {
        unsigned int sval = reinterpret_cast<unsigned int &>(value);
		encode(obuf, sval);
    }

    struct IntelOnIntelIO: BasicIO {
        virtual short to_int16(unsigned char const *) const;
        virtual unsigned short to_uint16(unsigned char const *) const;
		unsigned int to_uint32(unsigned char const *inbuf) const;
		int to_int32(unsigned char const *inbuf) const;
        virtual float to_float(unsigned char const *) const;
        virtual void encode(unsigned char *, short) const;
        virtual void encode(unsigned char *, unsigned short) const;
        virtual void encode(unsigned char *, float) const;
    };
    short IntelOnIntelIO::to_int16(unsigned char const *inbuf) const
    {
        char buf[2];
        std::copy(inbuf, inbuf+2, buf);
        short result = *reinterpret_cast<short *>(&buf);
        return result;
    }
    unsigned short IntelOnIntelIO::to_uint16(unsigned char const *inbuf) const
    {
        char buf[2];
        std::copy(inbuf, inbuf+2, buf);
        unsigned short result = *reinterpret_cast<unsigned short *>(&buf);
        return result;
    }
    float IntelOnIntelIO::to_float(unsigned char const *inbuf) const
    {
        char buf[4];
        std::copy(inbuf, inbuf+4, buf);
        float result = *reinterpret_cast<float *>(&buf);
        return result;
    }
    void IntelOnIntelIO::encode(unsigned char *obuf, short value) const
    {
        short sval = static_cast<short>(value);
        unsigned char const *bufp = reinterpret_cast<unsigned char *>(&sval);
        std::copy(bufp, bufp+2, obuf);
    }
    void IntelOnIntelIO::encode(unsigned char *obuf, unsigned short value) const
    {
        unsigned short sval = static_cast<unsigned short>(value);
        unsigned char const *bufp = reinterpret_cast<unsigned char *>(&sval);
        std::copy(bufp, bufp+2, obuf);
    }
    void IntelOnIntelIO::encode(unsigned char *obuf, float value) const
    {
        unsigned char const *bufp = reinterpret_cast<unsigned char *>(&value);
        std::copy(bufp, bufp+4, obuf);
    }
    int IntelOnIntelIO::to_int32(unsigned char const * /* inbuf */ ) const
	{
		throw UuIcsC3d::NotImplemented();
	}
    unsigned IntelOnIntelIO::to_uint32(unsigned char const * /* inbuf */) const
	{
		throw UuIcsC3d::NotImplemented();
	}

    boost::shared_ptr<BasicIO> intel_io(new IntelIO());
    //boost::shared_ptr<BasicIO> intel_io(new IntelOnIntelIO());

}

void BasicIO::encode(unsigned char *, int) const
{throw NotImplemented();}

void BasicIO::encode(unsigned char *, unsigned) const
{throw NotImplemented();}

void BasicIO::encode_array(unsigned char *, int const*, int) const
{throw NotImplemented();}

void BasicIO::encode_array(unsigned char *, unsigned const *, int) const
{throw NotImplemented();}


boost::shared_ptr<BasicIO> UuIcsC3d::get_basic_io(int pt)
{
	assert(sizeof(unsigned short)==2);
	assert(sizeof(unsigned)==4);
	assert(sizeof(float)==4);
    switch (pt) 
    {
    case Pt_Intel:
        return intel_io;
    case Pt_Dec:
        throw NotImplemented();
    case Pt_Mips:
        throw NotImplemented();
    default:
        throw UuIcsC3d::IllegalValue();
    }
}

UuIcsC3d::EncodingType UuIcsC3d::platform_encoding()
{
	return Pt_Intel;
}

boost::shared_ptr<BasicIO> UuIcsC3d::get_native_io()
{
	return intel_io;
}
