// view_c3d_skeleton.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "atlbase.h"
#include "Windowsx.h"
#include "view_c3d_skeleton.h"
#include <string>
#include "skeleton_info.hpp"
#include "uuc3d.hpp"
#include <iostream>
#include "subjects.hpp"


using std::vector;
using std::cerr;
using UuIcsC3d::SpacePaddedString;
using UuIcsC3d::FrameData;
using UuIcsC3d::LabeledSubject;

UuIcsC3d::SkeletonInfo const &get_uu_skeleton_info();

using std::string;

#define MAX_LOADSTRING 100

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

HDC   ghDC; 
HGLRC ghRC; 
HWND mainWnd;
HWND viewingWnd = NULL;  // Window handle of dialog box 

//vector<LabeledSubject> subjects;
vector<FrameData> frame_data;
unsigned cur_frame=0;
UINT animation_timer=1;
int camera=0;

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

void initializeGL(LONG width, LONG height);
void resize( LONG width, LONG height );
void drawScene(HDC const &ghDC, int camera, FrameData const *frame_data);
bool load_file1(string const &infile);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_VIEW_C3D_SKELETON, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_VIEW_C3D_SKELETON));

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (int) msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VIEW_C3D_SKELETON));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_VIEW_C3D_SKELETON);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	return RegisterClassEx(&wcex);
}

VOID CALLBACK MyTimerProc( 
    HWND hwnd,        // handle to window for timer messages 
    UINT message,     // WM_TIMER message 
    UINT idTimer,     // timer identifier 
    DWORD dwTime)     // current system time 
{
	size_t sz = frame_data.size();
	if (sz<2)
		return;
	++cur_frame;
	if (cur_frame >= sz)
		cur_frame=0;
	InvalidateRgn(hwnd,0,false);   
	UpdateWindow(hwnd);
}

//
//   FUNCTION: InitInstance(HINSTANCE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   //HWND hWnd;

   hInst = hInstance; // Store instance handle in our global variable

   mainWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!mainWnd)
   {
      return FALSE;
   }

   ShowWindow(mainWnd, nCmdShow);
   UpdateWindow(mainWnd);

   return TRUE;
}

void start_animation(bool synch=true)
{
	SetTimer(mainWnd,      // handle to main window 
		animation_timer,            // timer identifier 
		10,                    // 10-second interval 
		(TIMERPROC) MyTimerProc); // timer callback
	if (synch) {
		CheckDlgButton(viewingWnd,IDC_CHECK1,BST_CHECKED );
	}
}

void stop_animation(bool synch=true)
{
	KillTimer(mainWnd, animation_timer);
	animation_timer = 0;
	if (synch) {
		CheckDlgButton(viewingWnd,IDC_CHECK1,BST_UNCHECKED );
	}
}


void open_file(HWND hwnd)
{
	OPENFILENAME ofn;       // common dialog box structure
	TCHAR szFile[260];       // buffer for file name

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFile = szFile;
	//
	// Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
	// use the contents of szFile to initialize itself.
	//
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = _T("All\0*.*\0C3D\0*.c3d\0");
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	// Display the Open dialog box. 

	if (GetOpenFileName(&ofn)==TRUE) {
		ATL::CT2AEX<> pszA( ofn.lpstrFile );
		if (load_file1(pszA.m_psz)) {
			stop_animation();
			cur_frame=0;
			InvalidateRgn(hwnd,0,false);   
			UpdateWindow(hwnd);
		}
	}
}

BOOL bSetupPixelFormat(HDC hdc) 
{ 
    PIXELFORMATDESCRIPTOR pfd, *ppfd; 
    int pixelformat; 
 
    ppfd = &pfd; 
 
    ppfd->nSize = sizeof(PIXELFORMATDESCRIPTOR); 
    ppfd->nVersion = 1; 
    ppfd->dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |  
                        PFD_DOUBLEBUFFER; 
    ppfd->dwLayerMask = PFD_MAIN_PLANE; 
    ppfd->iPixelType = PFD_TYPE_RGBA; 
    ppfd->cColorBits = 24;
	ppfd->cRedBits = 8;
	ppfd->cRedShift = 0;
	ppfd->cGreenBits = 8;
	ppfd->cGreenShift = 0;
	ppfd->cBlueBits = 8;
	ppfd->cBlueShift = 0;
    ppfd->cDepthBits = 16; 
    ppfd->cAccumBits = 0; 
    ppfd->cStencilBits = 0; 
 
    pixelformat = ChoosePixelFormat(hdc, ppfd); 
 
    if ( (pixelformat = ChoosePixelFormat(hdc, ppfd)) == 0 ) 
    { 
        MessageBox(NULL, _T("ChoosePixelFormat failed"), _T("Error"), MB_OK); 
        return FALSE; 
    } 
 
    if (SetPixelFormat(hdc, pixelformat, ppfd) == FALSE) 
    { 
        MessageBox(NULL, _T("SetPixelFormat failed"), _T("Error"), MB_OK); 
        return FALSE; 
    } 
 
    return TRUE; 
} 

BOOL CALLBACK ViewingProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{ 
	//BOOL fError; 

	switch (message) 
	{ 
	case WM_INITDIALOG: 
		{
		HWND combo = GetDlgItem(hwndDlg, IDC_CAMCOMBO);
		ComboBox_AddString(combo, _T("Camera 1"));
		ComboBox_AddString(combo, _T("Camera 2"));
		ComboBox_AddString(combo, _T("Camera 3"));
		ComboBox_AddString(combo, _T("Camera 4"));
		ComboBox_AddString(combo, _T("Camera 5"));
		ComboBox_AddString(combo, _T("Camera 6"));
		ComboBox_AddString(combo, _T("Camera 7"));
		ComboBox_AddString(combo, _T("Camera 8"));
		ComboBox_SetCurSel(combo,0);
		return TRUE; 
		}
	case WM_CLOSE:
		ShowWindow(hwndDlg, SW_HIDE); 
		return TRUE;
	case WM_COMMAND: 
		switch (LOWORD(wParam)) 
		{ 
		case IDC_CAMCOMBO:
			camera=ComboBox_GetCurSel(GetDlgItem(hwndDlg, IDC_CAMCOMBO));
			InvalidateRgn(mainWnd,0,false);   
			UpdateWindow(mainWnd);

		case IDC_CHECK1:
			if (IsDlgButtonChecked(hwndDlg, IDC_CHECK1)==BST_CHECKED)
				start_animation(false);
			else
				stop_animation(false);
			break;
		case IDM_EXIT:
			DestroyWindow(hwndDlg);
			return TRUE;
			break;
		} 
	} 
	return FALSE; 
} 

 //
//  FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
	RECT rect;

	switch (message)
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		// Parse the menu selections:
		switch (wmId)
		{
		case IDM_ABOUT:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUTBOX), hWnd, About);
			break;
		case ID_FILE_OPEN:
			open_file(hWnd); 
			break;
		case ID_FILE_VIEWING:
		case ID_VIEW_VIEW:
            if (!IsWindow(viewingWnd)) 
            { 
                viewingWnd = CreateDialog(hInst, 
                                        MAKEINTRESOURCE(IDD_VIEWING_DIALOG), 
                                        hWnd, 
                                        (DLGPROC)ViewingProc); 
			}
            ShowWindow(viewingWnd, SW_SHOW); 
			break;
		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
    case WM_CREATE: 
        ghDC = GetDC(hWnd); 
        if (!bSetupPixelFormat(ghDC)) 
            PostQuitMessage (0); 
 
        ghRC = wglCreateContext(ghDC); 
        wglMakeCurrent(ghDC, ghRC); 
        GetClientRect(hWnd, &rect); 
        initializeGL(rect.right, rect.bottom); 
        break;
    case WM_SIZE: 
        GetClientRect(hWnd, &rect); 
        resize(rect.right, rect.bottom); 
        break; 
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		if (cur_frame<frame_data.size()) {
			drawScene(ghDC, camera, &frame_data[cur_frame]);
		} else {
			drawScene(ghDC, camera, 0);
		}
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(lParam);
	switch (message)
	{
	case WM_INITDIALOG:
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}


bool load_file1(string const &infile)
{
	// TODO: handle exceptions properly
	try {
		UuIcsC3d::C3dFileInfo fi(infile);
		int ppf = fi.points_per_frame(); 
		if (ppf <= 0)
			return false;
		int fc = fi.frame_count();
		vector<SpacePaddedString> labels(fi.point_labels());
//		vector<LabeledSubject> new_subjects;
//		vector<unsigned> no_label = fill_subjects(
//			new_subjects, labels, get_uu_skeleton_info());
		for (int i=labels.size(); i<ppf; ++i) {
			labels.push_back(SpacePaddedString("<no label>"));
		}
		std::auto_ptr<UuIcsC3d::C3dFile> filep(fi.open());
		vector<FrameData> new_frame_data(fc);
		for (int i=0; i<fc; ++i) {
			filep->get_frame_data(new_frame_data[i], i);
		}
		filep.reset();
//		subjects.swap(new_subjects);
		frame_data.swap(new_frame_data);
		cur_frame=0;
	} catch (UuIcsC3d::ContentError const &ce) {
		string msg = ce.msg();
	} catch(...) {
		return false;
	}
	return true;
}
